@@ -58,6 +58,7 @@ namespace Hipo
 		private Gtk.Menu menu;
 		private Hashtable menuTable;
 		private Gtk.Image coverpreview;
+		private ProgressDialog pdialog;
 		
 		ActionGroup group = null;
 			 
@@ -73,7 +74,7 @@ namespace Hipo
 		{
 			program = new Program (Defines.PACKAGE, Defines.VERSION, Modules.UI, args);
 
-			gxml = new Glade.XML (null, "hipo.glade", "hipoWindow", Defines.PACKAGE);
+			gxml = new Glade.XML (null, "hipo.glade", "hipoWindow", null);
 			gxml.Autoconnect (this);
 
 			combo = new DeviceCombo ();
@@ -102,7 +103,7 @@ namespace Hipo
 				new ActionEntry ("Eject", null, Catalog.GetString ("_Eject"), null, null, OnEjectActivated),
 				new ActionEntry ("Quit", Gtk.Stock.Quit, Catalog.GetString ("_Quit"), null, null, OnQuitActivated),
 				new ActionEntry ("Device", Gtk.Stock.Preferences, Catalog.GetString ("Prefere_nces"), null,
-						 null, OnDeviceProperties),
+						 null, OnPreferences),
 				new ActionEntry ("Help", Gtk.Stock.Help, Catalog.GetString ("_Contents"), "F1",
 						 null, OnHelp),
 				new ActionEntry ("About", Gnome.Stock.About, Catalog.GetString ("_About"), null,
@@ -111,7 +112,10 @@ namespace Hipo
 				new ActionEntry ("NewPlaylist", Gtk.Stock.New, Catalog.GetString ("New Playlist"), null,
 						 Catalog.GetString ("Create a new Playlist"), OnNewPlaylist),
 				new ActionEntry ("DeletePlaylist", Gtk.Stock.Remove, Catalog.GetString ("Delete Playlist"), null,
-						 Catalog.GetString ("Delete the Playlist"), OnDeletePlaylist)
+						 Catalog.GetString ("Delete the Playlist"), OnDeletePlaylist),
+
+				new ActionEntry ("OnDeviceProperties", Gtk.Stock.Properties, Catalog.GetString ("Device Properties"), null,
+						 Catalog.GetString ("Device Properties"), OnDeviceProperties)
 			};
 
 			group = new ActionGroup ("Hipo");
@@ -147,7 +151,8 @@ namespace Hipo
 							  tracksview_source_entries, Gdk.DragAction.Copy);
 
 			tracksView.DragDataGet += OnTracksDragDataGet;
-						
+			tracksView.Selection.Mode = SelectionMode.Multiple;
+			
 			plistView.ButtonPressEvent += OnPlaylistButtonPress;
 			plistView.PopupMenu += OnPlaylistPopupMenu;
 			
@@ -160,7 +165,9 @@ namespace Hipo
 			
 			menuTable = new Hashtable ();
 			coverpreview = new Gtk.Image ();
-			
+
+			pdialog = new ProgressDialog (hipoWindow);
+
 			hipoWindow.ShowAll ();
 
 			SetDevice ();
@@ -221,6 +228,7 @@ namespace Hipo
 			case 3:
 				if (this.device != null) {
 					TracksPopupMenu (args.Event);
+					args.RetVal = true;
 				}
 				break;
 			}
@@ -286,7 +294,15 @@ namespace Hipo
 
 					menupl.Popup (null, null, null, (ev != null) ? ev.Button : 0,
 						      (ev != null) ? ev.Time : 0);
-				}
+				} else {
+					if (plistName == device.Name) {
+						menupl = (Gtk.Menu) uim.GetWidget ("/ui/DevicePopup");
+						menupl.ShowAll ();
+
+						menupl.Popup (null, null, null, (ev != null) ? ev.Button : 0,
+							      (ev != null) ? ev.Time : 0);
+					}
+				}	
 			}
 		}
 
@@ -307,9 +323,13 @@ namespace Hipo
 				Playlist plist = this.db.LookupPlaylist (plistName);
 				this.db.RemovePlaylist (plist);
 
-				Thread thr = new Thread (new ThreadStart (SaveDB));
-				thr.Start ();
+				RefreshPlaylistView ();
+				playlists.RefreshDevice (this.device);
+				playlists.RemovePlaylist (iter);
 			}
+
+			Thread thr = new Thread (new ThreadStart (SaveDB));
+			thr.Start ();
 		}
 
 
@@ -365,10 +385,12 @@ namespace Hipo
 		{
 			if (device == null)
 			{
-				ipodCapacityBar.Text = Catalog.GetString ("No iPod Found");
+				SetGreyedObjects (false);
 				return;
 			}
 			
+			SetGreyedObjects (true);
+			
 			this.device = device;
 
 			Thread thr = new Thread (new ThreadStart (PopulateView));
@@ -384,11 +406,28 @@ namespace Hipo
 				Console.Error.WriteLine (e);
 			}
 		}
+
+		private void SetGreyedObjects (bool greyed)
+		{
+			if (!greyed) {
+				ipodCapacityBar.Text = Catalog.GetString ("No iPod Found");
+				ipodCapacityBar.Fraction = 0.0;
+				playlists.Clear ();
+				tracks.Clear ();
+			}
+
+			group["AddFile"].Sensitive = greyed;
+			group["AddFolder"].Sensitive = greyed;
+			group["Remove"].Sensitive = greyed;
+			group["Eject"].Sensitive = greyed;
+			group["NewPlaylist"].Sensitive = greyed;
+		}
 		
 		private void PopulateView ()
 		{
 			this.db = this.device.TrackDatabase;
-
+			pdialog.TrackDatabase = this.db;
+			
 			playlists.AddDeviceToStore (this.device);
 			
 			foreach (Playlist plist in this.db.Playlists)
@@ -487,6 +526,9 @@ namespace Hipo
 				tracks.AddTrack (this.db, uri.LocalPath);
 			}
 
+			Thread thr = new Thread (new ThreadStart (SaveDB));
+			thr.Start ();
+
 			Gtk.Application.Invoke (delegate {
 					statusbar1.Push (1, "");
 				});
@@ -505,58 +547,110 @@ namespace Hipo
 			TreeIter iter;
 			TreeModel model;
 
-			if (tracksView.Selection.GetSelected (out model, out iter))
+			Array treePaths = tracksView.Selection.GetSelectedRows (out model);
+
+			foreach (TreePath tPath in treePaths)
 			{
-				long trackId = (long) model.GetValue (iter, 4);
-				
-				foreach (Track track in this.db.Tracks)
+				if (model.GetIter (out iter, tPath))
 				{
-					if (track.Id == trackId)
-						playlist.AddTrack (track);
+					long trackId = (long) model.GetValue (iter, 4);
+				
+					foreach (Track track in this.db.Tracks)
+					{
+						if (track.Id == trackId)
+							playlist.AddTrack (track);
+					}
 				}
-
-				playlists.UpdateTrackNumber (this.db, playlist);
-
-				Thread thr = new Thread (new ThreadStart (SaveDB));
-				thr.Start ();
 			}
+
+			Thread thr = new Thread (new ThreadStart (SaveDB));
+			thr.Start ();
+
+			playlists.UpdateTrackNumber (this.db, playlist);
 		}
 		
 	      	private void OnRemoveActivated (System.Object o, EventArgs args)
 		{
 			TreeIter iter;
 			TreeModel model;
- 
-			if (tracksView.Selection.GetSelected (out model, out iter))
+			ListStore store;
+			Gtk.HBox hbox;
+			Gtk.ScrolledWindow scroll;
+			TreeView view;
+			Queue iterQueue;
+
+			iterQueue = new Queue ();
+			hbox = new Gtk.HBox ();
+			hbox.BorderWidth = 5;
+
+			scroll = new Gtk.ScrolledWindow ();
+			scroll.ShadowType = ShadowType.In;
+			scroll.VscrollbarPolicy = PolicyType.Automatic;
+			scroll.HscrollbarPolicy = PolicyType.Automatic;
+
+			store = new Gtk.ListStore (typeof (string));
+			view = new Gtk.TreeView ();
+			view.AppendColumn ("Songs", new Gtk.CellRendererText (), "text", 0);
+
+			Array treePaths = tracksView.Selection.GetSelectedRows (out model);
+			
+			MessageDialog dialog = new MessageDialog (hipoWindow,
+								  DialogFlags.DestroyWithParent,
+								  MessageType.Question,
+								  ButtonsType.OkCancel,
+								  String.Format ("<b>{0}</b>",
+										 Catalog.GetString ("Do you really want to remove the song/s?"))
+				);
+			
+			dialog.Resizable = true;
+			
+			foreach (TreePath tPath in treePaths)
 			{
-				string Artist = (string) model.GetValue (iter, 0);
-				string Album = (string) model.GetValue (iter, 1);
-				string Title = (string) model.GetValue (iter, 2);
-				long trackId = (long) model.GetValue (iter, 4);
-				
-				MessageDialog md = new MessageDialog (hipoWindow,
-								      DialogFlags.DestroyWithParent,
-								      MessageType.Question,
-								      ButtonsType.OkCancel,
-								      String.Format ("<b>{0}</b>",
-										     Catalog.GetString ("Do you really want to remove the song?"))
-					);
-				
-				
-				md.SecondaryText = String.Format (Catalog.GetString ("Artist: {0}\nAlbum: {1}\nTitle: {2}\n"),
-								  Artist, Album, Title);
-
-				ResponseType res = (ResponseType) md.Run ();
-
-				md.Destroy ();
-				
-				if (res == ResponseType.Ok)
-				{
-					tracks.RemoveTracks (this.db, trackId, iter);
-					RefreshPlaylistView  ();
-					playlists.RefreshDevice (this.device);
+				if (model.GetIter (out iter, tPath)) {
+					
+					string Artist = (string) model.GetValue (iter, 0);
+					string Album = (string) model.GetValue (iter, 1);
+					string Title = (string) model.GetValue (iter, 2);
+					
+					store.AppendValues (String.Format ("{0} - {1} - {2}", Artist, Album, Title));
 				}
 			}
+
+			view.Model = store;
+
+			scroll.Add (view);
+			hbox.Add (scroll);
+			hbox.ShowAll ();
+			dialog.VBox.Add (hbox);
+			
+			ResponseType response = (ResponseType) dialog.Run ();
+			dialog.Destroy ();
+			
+			if (response == ResponseType.Ok)
+			{
+				foreach (TreePath tPath in treePaths)
+				{
+					if (model.GetIter (out iter, tPath)) {
+						iterQueue.Enqueue (iter);
+						
+						long trackId = (long) model.GetValue (iter, 4);
+						tracks.RemoveTracks (this.db, trackId);
+					}
+                                }
+
+				while (iterQueue.Count > 0)
+				{
+					TreeIter iterator = (TreeIter) iterQueue.Dequeue ();
+					tracks.RemoveTracks (iterator);
+				}
+
+				lock (this.db)
+					this.db.Save ();
+
+				RefreshPlaylistView ();
+				playlists.RefreshDevice (this.device);
+
+			}
 		}
 
 		[Widget] private Gtk.Dialog trackProperties;
@@ -570,7 +664,10 @@ namespace Hipo
 		
 		private void OnTrackProperties (object o, EventArgs args)
 		{
-			Glade.XML xml = new Glade.XML (null, "hipo.glade", "trackProperties", Defines.PACKAGE);
+			if (tracksView.Selection.CountSelectedRows() > 1)
+				return;
+
+			Glade.XML xml = new Glade.XML (null, "hipo.glade", "trackProperties", null);
 			xml.Autoconnect (this);
 
 			bool modified = false;
@@ -579,7 +676,9 @@ namespace Hipo
 			TreeModel model;
 			Gdk.Pixbuf newCover = null;
 			
-			if (tracksView.Selection.GetSelected (out model, out iter))
+			Array treePaths = tracksView.Selection.GetSelectedRows (out model);
+
+			if (model.GetIter (out iter, (Gtk.TreePath) treePaths.GetValue(0)))
 			{
 				long trackId = (long) model.GetValue (iter, 4);
 				
@@ -633,9 +732,9 @@ namespace Hipo
 				fileChooser.Destroy ();
 			};
 
-			ResponseType res = (ResponseType) trackProperties.Run ();
+			ResponseType response = (ResponseType) trackProperties.Run ();
 				
-			if (res == ResponseType.Ok)
+			if (response == ResponseType.Ok)
 			{			
 				if (artistEntry.Text != track.Artist) {
 					modified = true;
@@ -684,6 +783,49 @@ namespace Hipo
 			trackProperties.Destroy ();
 		}
 
+		[Widget] Gtk.Dialog deviceProperties;
+		[Widget] Gtk.Label serialLabel;
+		[Widget] Gtk.Label modelLabel;
+		[Widget] Gtk.Label generationLabel;
+		[Widget] Gtk.Label capacityLabel;
+		[Widget] Gtk.Label mountLabel;
+		[Widget] Gtk.Label spaceLabel;
+		[Widget] Gtk.Label firmwareLabel;
+		[Widget] Gtk.Entry nameEntry;
+		
+		private void OnDeviceProperties (object o, EventArgs args)
+		{
+			Glade.XML xml = new Glade.XML (null, "hipo.glade", "deviceProperties", null);
+			xml.Autoconnect (this);
+
+			deviceProperties.Title = String.Format (Catalog.GetString ("{0} Properties"), this.device.Name);
+			nameEntry.Text = this.device.Name;
+
+			serialLabel.Text = this.device.SerialNumber;
+			modelLabel.Text = this.device.ModelString;
+			generationLabel.Text = String.Format ("{0}", this.device.Generation);
+			capacityLabel.Text = FormatString ((double) this.device.VolumeSize);
+			mountLabel.Text = this.device.MountPoint;
+			spaceLabel.Text = String.Format ("{0}%", (int) ((double) device.VolumeUsed / (double) device.VolumeSize * 100));
+			firmwareLabel.Text = this.device.FirmwareVersion;
+
+
+			ResponseType res = (ResponseType) deviceProperties.Run ();
+				
+			if (res == ResponseType.Ok)
+			{
+				if (this.device.Name != nameEntry.Text.Trim ())
+				{
+					this.device.Name = nameEntry.Text;
+					
+					SaveDB ();
+					playlists.RefreshDevice (this.device);
+				}
+			}
+
+			deviceProperties.Destroy ();
+		}
+
 		private void OnUpdatePreview (object o, EventArgs args)
 		{
 			try {
@@ -721,31 +863,46 @@ namespace Hipo
 
 		private void OnRemoveFromPlaylist (object o, EventArgs args)
 		{
-			TreeIter iter;
 			TreeModel model;
- 
-			if (tracksView.Selection.GetSelected (out model, out iter))
+			TreeModel modelpl;
+			TreeIter iterpl;
+			Playlist plist = null;
+
+			Queue iterQueue = new Queue ();
+
+			Array treePaths = tracksView.Selection.GetSelectedRows (out model);
+			
+			if (plistView.Selection.GetSelected (out modelpl, out iterpl))
 			{
-				long trackId = (long) tracksView.Model.GetValue (iter, 4);
-				TreeIter iterpl;
-				TreeModel modelpl;
-				Playlist plist;
+				string playlistName = (string) plistView.Model.GetValue (iterpl, 1);
+				plist = this.db.LookupPlaylist (playlistName);
+			}
 
-				if (plistView.Selection.GetSelected (out modelpl, out iterpl))
+			foreach (TreePath tPath in treePaths)
+			{
+				TreeIter iter;
+
+				if (model.GetIter (out iter, tPath))
 				{
-					string playlistName = (string) plistView.Model.GetValue (iterpl, 1);
-					
-					if ((plist = this.db.LookupPlaylist (playlistName)) != null)
-					{
-						plist.RemoveTrack (tracks.GetTrackbyId (this.db, trackId));
-						playlists.RemoveFromPlaylist (iter);
-						playlists.UpdateTrackNumber (this.db, plist);
+					long trackId = (long) model.GetValue (iter, 4);
+					iterQueue.Enqueue (iter);
 
-						Thread thr = new Thread (new ThreadStart (SaveDB));
-						thr.Start ();
-					}	
+					Track t = tracks.GetTrackbyId (this.db, trackId);
+
+					if (t != null)
+						plist.RemoveTrack (t);
 				}
 			}
+
+			while (iterQueue.Count > 0)
+			{
+				TreeIter iter = (TreeIter) iterQueue.Dequeue ();
+				playlists.RemoveFromPlaylist (iter);
+			}
+
+			SaveDB ();
+
+			playlists.UpdateTrackNumber (this.db, plist);
 		}
 
 		private void OnDeleteEvent (object o, DeleteEventArgs args)
@@ -778,7 +935,7 @@ namespace Hipo
 			}
 		}
 
-		private void OnDeviceProperties (object o, EventArgs args)
+		private void OnPreferences (object o, EventArgs args)
 		{
 			// FIXME
 		}
@@ -791,7 +948,8 @@ namespace Hipo
 		private void OnAboutActivated (object o, EventArgs args)
 		{
 			Gtk.AboutDialog aboutDialog = new Gtk.AboutDialog ();
-			string[] authors = { "Pedro Villavicencio Garrido <pvillavi@gnome.org>" };
+			string[] authors = { "Pedro Villavicencio Garrido <pvillavi@gnome.org>",
+					     "Felipe Barros S. <felipeb@gnome.cl>" };
 			string[] artists = { "Carlos Candia Flores <cerealk@vtr.net>",
 					     "Gabriel Bustos Farret  <lemut@lemut.com>"	};
 			
@@ -820,56 +978,21 @@ namespace Hipo
 			string used = null;
 			double percent;
 
-			capacity = FormatString (this.device.VolumeSize);
-			used = FormatString (this.device.VolumeUsed);
+			capacity = FormatString ((double) this.device.VolumeSize);
+			used = FormatString ((double) this.device.VolumeUsed);
 			percent = (System.Convert.ToDouble (used) * 100) / System.Convert.ToDouble (capacity);
 
 			ipodCapacityBar.Text = String.Format (Catalog.GetString ("Used {0} of {1}"), used, capacity);
 			ipodCapacityBar.Fraction = (percent * 1.0) / 100;
 		}
 
-		private string FormatString (ulong value)
+		private string FormatString (double value)
 		{
-			ulong kilo = 1024;
-			ulong mega = kilo * 1024;
-			ulong giga = mega * 1024;
-			string ret = null;
-			ulong total, resto;
-
-			if (value < kilo) {
-				ret = InsertSeparator (value) + " B";
-			}
-
-			if (value < mega) {
-				total = value / kilo;
-				resto = (value * 100 / kilo) % 100;
-				ret = InsertSeparator (total) + "." + resto + " KB";
-			}
-
-			if (value < giga) {
-				total = value / mega;
-				resto = (value * 100 / mega) % 100;
-				ret = InsertSeparator (total) + "." + resto + " MB";
+			if (value < 1073741824) {
+				return String.Format ("{0:0.0} MB", (value / 1048576));
 			} else {
-				if (value >= giga) {
-					total = value / giga;
-					resto = (value * 100 / giga) % 100;
-					ret = InsertSeparator (total) + "." + resto + " GB";
-				}
+				return String.Format ("{0:0.0} GB", (value / 1073741824));
 			}
-
-			return ret;
-		}
-
-		private string InsertSeparator (ulong number)
-		{
-			string str = number.ToString ();
-
-    			for (int i = str.Length-3; i > 0; i -= 3) {
-				str.Insert (i, ",");
-			}
-
-			return (str);
 		}
 
 		public static Gdk.Pixbuf GetDeviceIcon (IPod.Device device)
